;;;; autoload.scm -- load modules lazily
;;
;; Copyright (c) 2005 Alex Shinn
;; All rights reserved.
;;
;; BSD-style license: http://www.debian.org/misc/bsd.license

;; Provides an Emacs-style autoload facility which takes the basic form
;;
;;   (autoload unit procedure-name ...)
;;
;; such that the first time procedure-name is called, it will perform a
;; runtime require of 'unit and then apply the procedure from the newly
;; loaded unit to the args it was passed.  Subsequent calls to
;; procedure-name will thereafter refer to the new procedure and will
;; thus not incur any overhead.
;;
;; You may also specify an alias for the procedure, and a default
;; procedure if the library can't be loaded:
;;
;;   (autoload unit (name alias default) ...)
;;
;; In this case, although the procedure name from the unit is "name,"
;; the form defines the autoload procedure as "alias."
;;
;; If the library can't be loaded then an error is signalled, unless
;; default is given, in which case the values are passed to that.
;;
;; Examples:
;;
;; ;; load iconv procedures lazily
;; (autoload iconv iconv iconv-open)
;;
;; ;; load some sqlite procedures lazily with "-" names
;; (autoload sqlite (sqlite:open sqlite-open)
;;                  (sqlite:execute sqlite-execute))
;;
;; ;; load md5 library, falling back on slower scheme version
;; (autoload scheme-md5 (md5:digest scheme-md5:digest))
;; (autoload md5 (md5:digest #f scheme-md5:digest))

(define-macro (autoload unit . procs)
  (cons 'begin
        (map
         (lambda (x)
           (let-optionals* (if (pair? x) x (list x))
               ((name #f) (alias #f) (default #f))
             (unless alias (set! alias name))
             (if default
               `(define (,alias . args)
                  (condition-case
                   (begin
                     (require ',unit)
                     (set! ,alias ,name)
                     (apply (eval ',name) args))
                   (exn ()
                     (set! ,alias ,default)
                     (apply (eval ,alias) args))))
               `(define (,alias . args)
                  (require ',unit)
                  (set! ,alias ,name)
                  (apply (eval ',name) args)))))
         procs)))

